@@ -18,7 +18,8 @@ sub resolve_query {
         print "doing $event \n";
         return $event;
     }
-    elsif ($query =~ /columns_priv/ || $query =~ /db/) {
+    elsif ($query =~ /columns_priv/ || $query =~ /db/ || $query =~ /^create/i
+            || $query =~ /^update/i || $query =~ /^delete/i) {
         return 'proxy_do';
     }
     return 'proxy_query';
@@ -39,8 +40,8 @@ sub proxy_do {
     };
 
     if ($@) {
-        $self->send_error($@); 
-    }	
+        $self->send_error($@);
+    }
 }
 
 sub proxy_query {
@@ -58,10 +59,10 @@ sub proxy_query {
 
         my $sth = $DBH->prepare($data);
         if (!$sth) {
-            $self->send_error($DBH->errstr); 
+            $self->send_error($DBH->errstr);
         }
         if (!$sth->execute) {
-            $self->send_error($sth->errstr); 
+            $self->send_error($sth->errstr);
         }
 
         my $names = $sth->{'NAME'};
@@ -69,23 +70,22 @@ sub proxy_query {
         for (my $i = 0;  $i < $numFields;  $i++) {
             push @headers, $$names[$i];
         }
-        
+
         $self->send_results(\@headers, $results);
     };
 
     if ($@) {
-        $self->send_error($@); 
-    }	
+        $self->send_error($@);
+    }
 }
 
 sub _connect {
     my ($self) = @_;
 
-    my $dsn = "DBI:mysql:database=".$ENV{mysql_db}.";host=127.0.0.1;port=3306";
+    my $dsn = "DBI:mysql:database=".$ENV{mysql_db}.";host=".$ENV{mysql_server}.";port=3306";
 
     $DBH = DBI->connect($dsn, $ENV{mysql_username}, $ENV{mysql_password});
 }
 
 
 1;
-
